from datetime import datetime
from glob import glob

import cv2
import numpy as np

now = datetime.now()
folder_path = r"depot_test\Test"
image_paths = sorted(glob(f"{folder_path}/*.png"))
image_files = [cv2.imread(p)[140:1000, :] for p in image_paths]

orb = cv2.ORB_create(nfeatures=1000)
FLANN_INDEX_LSH = 6
index_params = dict(
    algorithm=FLANN_INDEX_LSH, table_number=6, key_size=12, multi_probe_level=1
)
search_params = dict(checks=20)
flann = cv2.FlannBasedMatcher(index_params, search_params)

key_desc = [
    orb.detectAndCompute(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY), None)
    for img in image_files
]

offset_xs = []
for i in range(len(image_files) - 1):
    kp1, des1 = key_desc[i]
    kp2, des2 = key_desc[i + 1]
    matches = flann.knnMatch(des1, des2, k=2)
    good = [
        m for m, n in matches if n.distance > 1e-6 and m.distance < 0.75 * n.distance
    ]

    src_pts = np.float32([kp1[m.queryIdx].pt for m in good]).reshape(-1, 1, 2)
    dst_pts = np.float32([kp2[m.trainIdx].pt for m in good]).reshape(-1, 1, 2)
    H, _ = cv2.findHomography(dst_pts, src_pts, cv2.RANSAC, 2.0)
    offset_x = int(round(cv2.perspectiveTransform(np.float32([[[0, 0]]]), H)[0, 0, 0]))

    offset_xs.append(offset_x)

absolute_xs = [0]
for dx in offset_xs:
    absolute_xs.append(absolute_xs[-1] + dx)

min_x = min(absolute_xs)
max_x = max(absolute_xs[i] + img.shape[1] for i, img in enumerate(image_files))
height = image_files[0].shape[0]
width = max_x - min_x

stitched = np.zeros((height, width, 3), dtype=np.uint8)

for img, abs_x in zip(image_files, absolute_xs):
    x_start = abs_x - min_x
    x_end = x_start + img.shape[1]
    stitched[:, x_start:x_end] = img

cv2.imwrite("depot_test/result_refined.png", stitched)
print(datetime.now() - now)
