import glob
import os
from datetime import datetime

import cv2


def stitch_panorama(input_dir, output_dir):
    os.makedirs(output_dir, exist_ok=True)
    image_paths = sorted(glob.glob(os.path.join(input_dir, "*.*")))
    images = []
    for p in image_paths:
        img = cv2.imread(p)
        if img is None:
            continue
        images.append(img[140:1000, :])  # 截取 Y=140–1000 的行

    if len(images) < 2:
        print("错误：图片不足，无法拼接。")
        return

    stitcher = cv2.Stitcher_create()
    status, pano = stitcher.stitch(images)
    if status == cv2.Stitcher_OK:
        out_path = os.path.join(output_dir, "Output_stitch.png")
        cv2.imwrite(out_path, pano)
        print(f"拼接成功，保存至：{out_path}")
    else:
        print(f"拼接失败，错误码：{status}")


if __name__ == "__main__":
    start = datetime.now()
    stitch_panorama(r"depot_test\Test", r"depot_test\output")
    print("总耗时：", datetime.now() - start)
