import glob
import os
from datetime import datetime

import cv2
import numpy as np

now = datetime.now()

input_dir = r"depot_test\Test"
output_stitched_path = r"depot_test\stitched_image_multi.png"
output_visualization_path = r"depot_test\stitched_visualization.png"
image_pattern = "*.png"

CROP_Y_START = 140
CROP_Y_END = 1000
TEMPLATE_WIDTH = 100
TEMPLATE_START_X = 400

MIN_MATCH_CONFIDENCE = 0.5

image_files = sorted(glob.glob(os.path.join(input_dir, image_pattern)))


match_data = []
image_cache = {}

img1_path = image_files[0]
# Start inline load_and_crop_image for img1
img1_raw = cv2.imread(img1_path)
img1 = img1_raw[CROP_Y_START:CROP_Y_END, :]
# End inline load_and_crop_image for img1
IMG_HEIGHT, IMG_WIDTH, _ = img1.shape
image_cache[img1_path] = img1


for i in range(len(image_files) - 1):
    current_img_path = image_files[i]
    next_img_path = image_files[i + 1]

    img_current = image_cache.get(current_img_path)
    if img_current is None:
        # Start inline load_and_crop_image for img_current
        img_current_raw = cv2.imread(current_img_path)
        img_current = img_current_raw[CROP_Y_START:CROP_Y_END, :]
        # End inline load_and_crop_image for img_current
        image_cache[current_img_path] = img_current

    img_next = image_cache.get(next_img_path)
    if img_next is None:
        # Start inline load_and_crop_image for img_next
        img_next_raw = cv2.imread(next_img_path)
        img_next = img_next_raw[CROP_Y_START:CROP_Y_END, :]
        # End inline load_and_crop_image for img_next
        image_cache[next_img_path] = img_next

    tx = TEMPLATE_START_X
    ty = 0
    tw = TEMPLATE_WIDTH
    th = IMG_HEIGHT

    template = img_next[ty : ty + th, tx : tx + tw]

    result = cv2.matchTemplate(img_current, template, cv2.TM_CCOEFF_NORMED)
    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)

    match_x, match_y = max_loc

    split_x_current = match_x
    split_x_next = tx

    match_data.append(
        {
            "img1_path": current_img_path,
            "img2_path": next_img_path,
            "split_x_img1": split_x_current,
            "split_x_img2": split_x_next,
            "confidence": max_val,
        }
    )


image_offsets = {image_files[0]: 0}
current_offset = 0

for i, match in enumerate(match_data):
    img1_path = match["img1_path"]
    img2_path = match["img2_path"]
    split_x_img1 = match["split_x_img1"]
    split_x_img2 = match["split_x_img2"]

    offset_img1 = image_offsets[img1_path]
    offset_img2 = offset_img1 + split_x_img1 - split_x_img2
    image_offsets[img2_path] = offset_img2

    current_offset = offset_img2

last_image_path = image_files[-1]
last_image_offset = image_offsets[last_image_path]
total_width = last_image_offset + IMG_WIDTH


stitched_img = np.zeros((IMG_HEIGHT, total_width, 3), dtype=np.uint8)

stitched_image_paths = sorted(image_offsets.keys(), key=lambda p: image_offsets[p])

for i, img_path in enumerate(stitched_image_paths):
    img = image_cache.get(img_path)
    # Assume img is loaded and dimensions are correct

    x_start = image_offsets[img_path]

    copy_start_x = 0
    if i > 0:
        # Assume match_data[i-1] corresponds to the transition to this image
        copy_start_x = match_data[i - 1]["split_x_img2"]

    copy_width = IMG_WIDTH - copy_start_x
    source_img_part = img[:, copy_start_x:]

    dest_start_x = x_start + copy_start_x
    dest_end_x = dest_start_x + copy_width

    # Assume indices and shapes are correct for pasting
    stitched_img[:, dest_start_x:dest_end_x] = source_img_part


cv2.imwrite(output_stitched_path, stitched_img)

print(datetime.now() - now)
