from mower.solvers.infra.base_mixin import BaseMixin
from mower.solvers.infra.enter_room import EnterRoomSolver
from mower.utils import config
from mower.utils.image import cmatch, crop2content, cropimg, loadres, thres2
from mower.utils.log import logger
from mower.utils.rapidocr import ocr_rec
from mower.utils.recognize import Scene
from mower.utils.solver import BaseSolver
from mower.utils.vector import va

from .utils import clue_cls, clue_scope


class GiveAwaySolver(BaseSolver, BaseMixin):
    solver_name = "传递线索"

    def run(self, clue_count) -> None:
        self.clue_count = clue_count
        if not (config.conf.leifeng_mode or self.clue_count > 9):
            return
        super().run()

    def transition(self) -> bool:
        if (
            scene := self.scene()
        ) == Scene.INFRA_DETAILS and self.detect_room() == "meeting":
            self.tap((500, 1000))
        elif scene == Scene.INFRA_CONFIDENTIAL:
            self.ctap((1799, 578))
        elif scene == Scene.CLUE_GIVE_AWAY:
            if not (config.conf.leifeng_mode or self.clue_count > 9):
                self.tap((1868, 54))
                return True
            if self.animation():
                return
            if (c := clue_cls("give_away")) is None:
                self.tap((1868, 54))
                return True
            if config.recog.gray[230][600] < 90:
                self.ctap(clue_scope["give_away"], 5)
                return
            if self.find("clue/icon_notification"):
                return
            if self.detect_product_complete():
                return
            orange = False
            for i in range(4):
                scope = (1285 + (c - 1) * 65, 137 + i * 225)
                bg = loadres("clue/give_away_digit_bg")
                h, w, _ = bg.shape
                scope = scope, va(scope, (w, h))
                img = cropimg(config.recog.img, scope)
                if cmatch(img, bg):
                    orange = True
                    break
            if not orange:
                i = 0
            name_top_left = (880, 125 + 225 * i)
            name_scope = (name_top_left, va(name_top_left, (390, 34)))
            name = cropimg(config.recog.gray, name_scope)
            name = thres2(name, 250)
            name = crop2content(name)
            name = ocr_rec(name)
            logger.info(f"给{name}送一张线索{c}")
            self.tap((1808, 200 + i * 255))
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            EnterRoomSolver().run("meeting", detail=False)
