from datetime import timedelta

from mower.solvers.infra.base_mixin import BaseMixin
from mower.solvers.infra.enter_room import EnterRoomSolver
from mower.utils import config
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver


class MessageBoard(BaseSolver, BaseMixin):
    solver_name = "留言板"
    solver_max_duration = timedelta(minutes=2)

    def transition(self):
        if (
            scene := self.scene()
        ) == Scene.INFRA_DETAILS and self.detect_room() == "meeting":
            if pos := self.find("clue/title_party"):
                self.tap(pos)
                return
            score, scope = config.recog.match("clue/message_board_entry")
            if score > 0.8:
                self.ctap(scope)
                return
            self.tap("clue/interact")
        elif scene == Scene.CLUE_MESSAGE_BOARD:
            if self.animation():
                return
            if pos := self.find("clue/message_board_collect"):
                self.tap(pos)
                return
            return True
        elif scene in [
            Scene.INFRA_CONFIDENTIAL,
            Scene.CLUE_MESSAGE_BOARD_FRIEND,
            Scene.CLUE_ACCESS_RECORD,
        ]:
            self.cback(3, id="infra_back")
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            EnterRoomSolver().run("meeting", detail=False)
