from datetime import timedelta

from mower.solvers.infra.base_mixin import BaseMixin
from mower.solvers.infra.enter_room import EnterRoomSolver
from mower.utils import config
from mower.utils.image import cmatch, cropimg, loadres
from mower.utils.log import logger
from mower.utils.recognize import Scene
from mower.utils.solver import BaseSolver
from mower.utils.vector import va


class PartyTimeSolver(BaseSolver, BaseMixin):
    solver_name = "线索交流结束时间"
    solver_max_duration = timedelta(minutes=2)

    def party_on(self) -> bool:
        res = loadres("clue/party_on")
        h, w, _ = res.shape
        top_left = 1900, 186
        scope = top_left, va(top_left, (w, h))
        img = cropimg(config.recog.img, scope)
        return cmatch(res, img)

    def transition(self) -> bool:
        if (
            scene := self.scene()
        ) == Scene.INFRA_DETAILS and self.detect_room() == "meeting":
            if self.animation(((1780, 200), (1910, 250))):
                return
            if not self.party_on():
                config.party_time = None
                logger.info("线索交流未开启")
                return True
            if self.find("clue/title_party"):
                config.party_time = self.double_read_time(((1768, 438), (1902, 480)))
                logger.info(f"线索交流结束时间：{config.party_time}")
                return True
            if pos := self.find("clue/show_party_details"):
                self.tap(pos)
                return
        elif scene == Scene.INFRA_CONFIDENTIAL:
            self.cback(3, id="infra_back")
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            EnterRoomSolver().run("meeting", detail=False)
