from datetime import datetime, timedelta
from typing import Literal

from mower.solvers.infra.base_mixin import BaseMixin
from mower.utils import config
from mower.utils.log import logger
from mower.utils.recognize import Scene
from mower.utils.solver import BaseSolver


class EnterActivitySolver(BaseSolver, BaseMixin):
    solver_name = "进入活动室"
    activity_scope = {
        1: ((1045, 410), (1144, 443)),
        2: ((1045, 569), (1144, 602)),
        3: ((1045, 728), (1144, 761)),
        4: ((1521, 410), (1620, 443)),
        5: ((1521, 569), (1620, 602)),
        6: ((1521, 728), (1620, 761)),
    }

    def run(self, num: Literal[1, 2, 3, 4, 5, 6]):
        """
        Args:
            num:活动室层数
        """
        logger.info(f"进入活动室{num}")
        self.num = num
        self.start_time = None
        self.success = False
        self.flag = False
        super().run()
        return self.success

    def timeout(self) -> bool:
        if self.start_time is None:
            self.start_time = datetime.now()
        return datetime.now() - self.start_time > timedelta(seconds=20)

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.INFRA_MAIN:
            if self.animation():
                return
            if self.timeout():
                self.back()
                self.start_time = None
                self.flag = False
                return
            if not self.find("meeting"):
                if self.flag:
                    return
                self.swipe_ext([(1820, 870), (100, 870)], [200], 600, 0.2)
                return
            if not self.find("activity", scope=self.activity_scope[self.num]):
                return True
            self.ctap(self.activity_scope[self.num], 3)
            self.flag = True

        elif scene == Scene.ACTIVITY_ROOM_DETAILS:
            if self.animation(ratio=0.2):
                return
            if config.recog.num.number_int("noto", ((780, 40), (800, 80)), 23, 100) != (
                (self.num - 1) % 3 + 1
            ):
                self.cback(3)
                return
            self.success = True
            return True

        else:
            self.scene_graph_step(Scene.INFRA_MAIN)
