import cv2
import numpy as np
from scipy.ndimage import maximum_filter

from mower.utils import typealias as tp
from mower.utils.image import loadres, template
from mower.utils.log import logger


class StagePosDetector:
    def get_stage_scope(self, res_name="operation") -> list[tp.Scope]:
        img = cv2.cvtColor(self.whole_scene, cv2.COLOR_RGB2GRAY)
        threshold = 0.7
        res = loadres(f"navigation/{res_name}", True)
        result = cv2.matchTemplate(img, res, cv2.TM_CCOEFF_NORMED, None)
        max_filter_size = 200
        local_max = maximum_filter(result, size=max_filter_size) == result
        matched_points = np.argwhere(local_max & (result > threshold))
        for pos in matched_points:
            y, x = pos
            self.stage_scope.append(((x - 50, y + 10), (x + 150, y + 70)))

    def detect_all_stage(self):
        result = {}
        img = cv2.cvtColor(self.whole_scene, cv2.COLOR_RGB2GRAY)
        for match_scope in self.stage_scope:
            max_score = 0
            max_scope = None
            for name in self.names:
                score, scope = template(img, self.names[name], match_scope)
                if score > max_score:
                    max_score = score
                    max_scope = scope
            result[name] = max_scope
        return result  # 有问题

    def all_stage_pos(self, names: dict[str, tp.Image], whole_scene: tp.Image):
        self.names = names
        self.whole_scene = whole_scene
        self.stage_scope = []
        self.get_stage_scope()
        for name in names:
            if "TR-" in name:
                self.get_stage_scope("tutorial")
                break
        logger.debug(f"{self.stage_scope=}")
        res = self.detect_all_stage()
        return res
