from typing import Literal

import cv2

from mower.utils import config
from mower.utils.image import cropimg, thres2
from mower.utils.log import logger
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver

from .utils import last_letters


class LastStageNavigation(BaseSolver):
    def run(
        self, name: str, mode: Literal["auto", "copy"] = "auto", switch_to_ex=False
    ) -> bool:
        self.switch_to_ex = switch_to_ex
        self.name = name
        self.mode = mode
        self.success = True
        if self.name != "":
            self.scene_graph_navigation(Scene.TERMINAL_MAIN)
        super().run()
        return self.success

    def last_stage(self) -> str:
        img = cropimg(config.recog.gray, ((1690, 845), (1870, 890)))
        dark_img = cv2.bitwise_not(img)
        dark_img = thres2(dark_img, 127)

        contours, _ = cv2.findContours(
            dark_img, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE
        )
        rect = [cv2.boundingRect(c) for c in contours]
        rect.sort(key=lambda c: c[0])

        value = ""
        for x, y, w, h in rect:
            if h < 7:
                if w < 7:
                    continue
                value += "-"
                continue
            digit = cropimg(img, ((x, y), (x + w, y + h)))
            digit = cv2.copyMakeBorder(digit, 10, 10, 10, 10, cv2.BORDER_REPLICATE)
            score = {}
            for letter, letter_img in last_letters.items():
                dh, dw = digit.shape
                lh, lw = letter_img.shape
                if dh < lh or dw < lw:
                    return ""
                result = cv2.matchTemplate(digit, letter_img, cv2.TM_SQDIFF_NORMED)
                min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
                score[letter] = min_val
            value += min(score, key=score.get)
        logger.debug(value)
        return value

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.TERMINAL_MAIN:
            if self.name in ["", self.last_stage()]:
                self.tap((1670, 880))
            else:
                self.success = False
                return True
        elif scene == Scene.OPERATOR_BEFORE:
            if self.find("ope_ex_mode") and self.switch_to_ex:
                self.ctap("ope_ex_mode", 3)
                return

            if self.mode == "auto":
                return True
            elif self.mode == "copy":
                if not self.switch_to_ex:
                    if config.recog.gray[65][1333] < 200:
                        self.sleep()
                        return
                    # 取消代理作战
                    if config.recog.gray[907][1600] > 127 and not self.find(
                        "ope_agency_lock"
                    ):
                        self.ctap((1776, 908), 3)
                        return
                    self.ctap("ope_start", 3)
                else:
                    self.ctap("ope_ex_start", 3)

        elif scene == Scene.OPERATOR_SELECT and self.mode == "copy":
            return True
        elif scene in [Scene.OPERATOR_RECOVER_ORIGINITE, Scene.OPERATOR_RECOVER_POTION]:
            return True

        else:
            self.scene_graph_step(Scene.TERMINAL_MAIN)
