import cv2

from mower.utils import config
from mower.utils.image import loadres
from mower.utils.log import logger
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver
from mower.utils.vector import va, vs


class Score(BaseSolver):
    solver_name = "曲谱"

    timeline = {
        "主题曲": {
            "0": (0, 0),
            "1": [530, 3],
            "2": [1055, -54],
            "3": [1540, 5],
            "4": [2206, 8],
            "5": [2800, -59],
            "6": [3335, -66],
            "7": [3973, -78],
            "8": [4497, -72],
            "9": [5201, -81],
            "10": [5819, -66],
            "11": [6376, -70],
            "12": [7014, -23],
            "13": [7544, -64],
            "14": [8199, -54],
        },
        "方舟": {},
        "那被祝福的": {},
        "山雪与银铁": {},
        "七丘的新芽": {},
        "霓虹之下": {},
        "岁岁今朝": {},
        "摘取未来之人": {},
        "自海渊的一瞥": {},
        "高塔迷影": {},
        "薪火重燃": {},
        "夏日律动": {
            "OF": (0, 0),
            "RI": [628, 18],
            "DH": [1239, 17],
            "IC": [1824, 19],
            "SL": [2439, 26],
            "AS": [3016, 44],
        },
        "泰拉奇谈": {},
    }

    def run(self, target: str):
        dash = target.rfind("-")
        self.prefix = target[0:dash]
        self.suffix = target[dash + 1 :]
        self.generates_imgs()
        return super().run()

    @property
    def group(self) -> str | None:
        for group_name, prefix_list in self.timeline.items():
            if self.prefix in prefix_list:
                return group_name

    def find_group(self) -> str | None:
        for key in self.timeline.keys():
            if self.find(f"navigation/score/small/{key}"):
                return key

    def generates_imgs(self):
        self.prefixes = {}
        for prefix in self.timeline[self.group]:
            self.prefixes[prefix] = loadres(
                f"navigation/score/timeline/{self.group}/{prefix}", True
            )

    def find_prefix(self):
        prefix, val, loc = None, 0, None
        for n, img in self.prefixes.items():
            result = cv2.matchTemplate(config.recog.gray, img, cv2.TM_CCOEFF_NORMED)
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
            if max_val > val:
                val = max_val
                loc = max_loc
                prefix = n
        logger.debug(f"在{loc}找到关卡{prefix}")
        target = va(
            vs(loc, self.timeline[self.group][prefix]),
            self.timeline[self.group][self.prefix],
        )
        if target[0] > 1920:
            self.swipe_ext([(1400, 540), (200, 540), (200, 800)], [200, 300], 0, 0.1)
        elif target[0] < 0:
            self.swipe_ext([(200, 540), (1400, 540), (1400, 800)], [200, 300], 0, 0.1)
        else:
            self.ctap(target, 5)

    def transition(self):
        if (scene := self.scene()) == Scene.TERMINAL_SCORE:
            if self.find("navigation/score/circle"):
                if self.find(f"navigation/score/large/{self.group}"):
                    self.find_prefix()
                    return
                self.ctap("navigation/score/circle", 5)
                return
            if pos := self.find(f"navigation/score/large/{self.group}"):
                self.ctap(pos, 5)
                return
            if pos := self.find(f"navigation/score/small/{self.group}"):
                self.ctap(pos, 5)
                return
            groups = list(self.timeline.keys())
            index = groups.index(self.group)
            group = self.find_group()
            if groups.index(group) < index:
                self.swipe_ext([(180, 750), (180, 300)], [300], 500)
                return
            self.swipe_ext([(180, 300), (180, 750)], [300], 500)
        elif scene == Scene.TERMINAL_PREVIOUS:
            self.ctap((1700, 924), 5)
        elif scene == Scene.OPERATOR_CHOOSE_LEVEL:
            return True
        else:
            self.scene_graph_step(Scene.TERMINAL_SCORE)
