from functools import cached_property

from mower.utils import config
from mower.utils.log import logger
from mower.utils.recognize import Scene
from mower.utils.solver import BaseSolver

from .utils.utils import find_text


class NodeOptionSelectSolver(BaseSolver):
    solver_name = "节点选项选择"

    @cached_property
    def options(self):
        for stage_name in self._option[config.conf.maa_rg_theme]:
            if find_text(
                stage_name, 36, scope=([165, 700], [165 + len(stage_name) * 50, 800])
            ):
                return self._option[config.conf.maa_rg_theme][stage_name]
        logger.error("未登记节点选项")
        return []

    @property
    def _option(self):
        return {
            "Sarkaz": {
                "相遇": ["它邀请对方共享猎物"],
                "似是而非": [self.似是而非],
                "在故事结束之后": [self.在故事结束之后],
                "虫卡兹！": ["这怎么可能嘛？"],
                "阴魂不散": ["问他为什么哭"],
                "戴冠式": ["兄长为妹妹戴上冠冕"],
                "高空坠物": ["找些帮手来把这里搬空", "从中寻找有用的物件"],
                "前瞻": ["留意当下"],
                "原初异途": ["一个方盒", "那是一只角，还是一把铳？"],
                "待诉说的真相": ["收集散落的思绪"],
                "解惑": ["他们的头顶都生出了光环", "萨科塔杀死了萨卡兹"],
            }
        }

    @property
    def 在故事结束之后(self):
        if config.conf.rogue.mode == 0:
            return "战士的长角生根发芽"
        return "时光的永恒在此定格"

    @property
    def 似是而非(self):
        if config.conf.rogue.mode == 0:
            return "向她们求援"
        return "远远躲开"

    def transition(self):
        if (scene := self.scene()) == Scene.ROGUE_NODE_OPTION_SELECT:
            if pos := self.find("rogue/node_check_to_do"):
                self.ctap(pos, 3)
                return
            for option in self.options:
                if pos := find_text(option, 36, scope=((1270, 70), (1920, 940))):
                    self.ctap(pos, 3)
                    return
        elif scene in self.waiting_scene:
            self.waiting_solver()
        elif scene == Scene.ROGUE_NODE_LOADING:
            self.scene_graph_step(Scene.ROGUE_MAIN)
        else:
            return True
