"""
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import datetime

from .box import Box
from .headhunting import Headhunting
from .orundum import Orundum
from .shop import Shop
from .sign_in import SignIn
from .special_access import SpecialAccess
from .task import Task


class SignInManager:
    solver_name = "签到管理器"
    scheduler_stop_time = None

    def run(self) -> bool:
        result = True
        solver_list = []
        now = datetime.now()
        if now < datetime(2025, 6, 9):
            solver_list += [SpecialAccess()]
        if now < datetime(2025, 5, 15):
            solver_list += [SignIn(), Orundum(), Headhunting()]
        if now < datetime(2025, 5, 29):
            solver_list += [Task(), Shop(), Box()]
        for solver in solver_list:
            solver.scheduler_stop_time = self.scheduler_stop_time
            result = result and solver.run()
        return result
