"""
Copyright (c) 2025 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import timedelta

from mower.utils.email import notify
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver

activity_name = "制铳师之夜"


class Box(BaseSolver):
    solver_name = "物资箱子领取"
    solver_max_duration = timedelta(minutes=2)

    def run(self):
        self.materiel_email = False
        self.finish_email = True
        return super().run()

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.MATERIEL:
            if self.materiel_email:
                self.materiel_email = False
                self.finish_email = False
                self.sleep()
                notify(f"{activity_name}箱子领取")
            self.scene_graph_step(Scene.INDEX)
        elif scene == Scene.ACTIVITY_MAIN:
            if pos := self.find("sign_in/box/entry"):
                self.ctap(pos, 5)
            else:
                notify(f"未找到{activity_name}箱子领取入口")
                return True
        elif scene == Scene.ACTIVITY_BOX:
            if self.find("sign_in/box/complete"):
                if self.finish_email:
                    notify(f"{activity_name}箱子已领完")
                return True
            if pos := self.find("sign_in/box/collect"):
                if self.ctap(pos, 5):
                    self.finish_email = False
                    self.materiel_email = True
        else:
            self.scene_graph_step(Scene.ACTIVITY_MAIN)
