"""
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import timedelta

from mower.utils.email import notify
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver


class Headhunting(BaseSolver):
    solver_name = "每日赠送单抽"
    solver_max_duration = timedelta(minutes=1)

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.HEADHUNTING:
            if pos := self.find("sign_in/headhunting/available"):
                self.tap(pos)
            else:
                notify("今日赠送单抽已抽完")
                return True
        elif scene == Scene.HEADHUNTING_FREE_CONFIRM:
            self.tap("double_confirm/main", x_rate=1)
        elif scene == Scene.HEADHUNTING_RESULT:
            notify("成功抽完赠送单抽")
            return True
        else:
            self.scene_graph_step(Scene.HEADHUNTING)
