"""
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import cv2

from mower.utils import config
from mower.utils.email import notify
from mower.utils.image import cropimg
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver
from mower.utils.vector import sa

activity_name = "月饼烘焙小教程·三"


class MoonFestival(BaseSolver):
    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.MATERIEL:
            self.sleep()
            notify(f"{activity_name}奖励领取")
            self.scene_graph_step(Scene.INDEX)
        elif scene in self.waiting_scene:
            self.waiting_solver()
        elif (scene := self.scene()) == Scene.MOON_FESTIVAL:
            if pos := self.find("sign_in/moon_festival/moon_cake"):
                self.tap(pos)
                return
            top_left = 530, 910
            img = cropimg(config.recog.hsv, (top_left, (1390, 1020)))
            img = cv2.inRange(img, (5, 180, 200), (15, 255, 255))
            contours, _ = cv2.findContours(
                img, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE
            )
            rect = [cv2.boundingRect(c) for c in contours]
            if len(rect) == 0:
                self.back()
                return
            rect.sort(key=lambda c: c[0])
            x, y, w, h = rect[0]
            scope = sa(((x, y), (x + w, y + h)), top_left)
            self.ctap(scope)
        else:
            return True
