"""
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import timedelta

from mower.utils.email import notify
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver

from .utils import index_entry

activity_name = "育婴圣堂许愿墙"


class Orundum(BaseSolver):
    solver_name = "幸运墙"
    solver_max_duration = timedelta(minutes=2)

    def run(self):
        self.in_progress = False
        return super().run()

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.INDEX:
            if pos := index_entry("sign_in/orundum/entry", activity_name):
                self.ctap(pos)
            else:
                notify(f"未找到{activity_name}签到入口")
                return True
        elif scene == Scene.ANNOUNCEMENT:
            if self.animation():
                return
            if not self.find("sign_in/orundum/banner"):
                self.scene_graph_step(Scene.INDEX)
                return
            if self.find("sign_in/orundum/complete"):
                notify(f"今日{activity_name}奖励已领完")
                return True
            for x in range(445, 1520, 213):
                if self.find("sign_in/orundum/choose"):
                    self.tap((x, 415))
                elif pos := self.find("sign_in/orundum/confirm"):
                    self.in_progress = True
                    self.ctap(pos)
                else:
                    self.sleep()
        elif scene == Scene.MATERIEL:
            if self.in_progress:
                self.sleep()
                notify(f"成功领取{activity_name}奖励")
                return True
            self.scene_graph_step(Scene.INDEX)
        else:
            self.scene_graph_step(Scene.INDEX)
