"""
Copyright (c) 2025 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import timedelta

from mower.utils.email import notify
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver

from .utils import index_entry


class SpecialAccess(BaseSolver):
    solver_name = "活动推荐月卡"
    solver_max_duration = timedelta(minutes=2)

    def run(self):
        self.materiel_email = False
        self.finish_email = True
        return super().run()

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.MATERIEL:
            if self.materiel_email:
                self.materiel_email = False
                self.finish_email = False
                self.sleep()
                notify(f"{self.solver_name}领取")
            self.scene_graph_step(Scene.INDEX)
        elif scene == Scene.INDEX:
            if pos := index_entry("sign_in/special_access/entry", self.solver_name):
                self.ctap(pos)
                return
            notify(f"未找到{self.solver_name}入口")
            return True
        elif scene == Scene.ACTIVITY_SPECIAL_ACCESS:
            if self.animation():
                return
            if pos := self.find("sign_in/special_access/collect"):
                if self.ctap(pos, 5):
                    self.materiel_email = True
                    self.finish_email = False
                return
            if self.finish_email:
                notify(f"{self.solver_name}奖励已领完")
            return True
        else:
            self.scene_graph_step(Scene.INDEX)
