"""
Copyright (c) 2025 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import timedelta

from mower.utils import config
from mower.utils.email import notify
from mower.utils.image import cropimg
from mower.utils.rapidocr import ocr_rec
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver

task_name = "众生行记"


class Task(BaseSolver):
    solver_name = "活动奖励领取"
    solver_max_duration = timedelta(minutes=2)

    def run(self):
        self.notify_task = False
        self.notify_operator = False
        self.notify_done = True
        return super().run()

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.ACTIVITY_TASK:
            if self.animation():
                return
            scope = ((1500, 230), (1700, 280))
            if "键" in ocr_rec(cropimg(config.recog.img, scope)):
                self.notify_task = True
                self.notify_operator = True
                self.notify_done = False
                self.ctap(scope)
                return
            if self.notify_done:
                notify(f"{task_name}任务奖励已领完")
            return True
        elif scene == Scene.MATERIEL:
            if self.notify_task:
                self.notify_task = False
                self.sleep()
                notify(f"{task_name}任务奖励领取")
            self.scene_graph_step(Scene.INDEX)
        elif scene in [Scene.RECRUIT_AGENT, Scene.RECRUIT_AGENT]:
            if self.notify_operator:
                self.notify_operator = False
                notify("活动干员领取")
            self.ctap((960, 540))
        else:
            self.scene_graph_step(Scene.ACTIVITY_TASK)
