from datetime import datetime, timedelta
from typing import List, Tuple

from pydantic import BaseModel

from mower.solvers.自动专精.A_加载协助者 import 协助者


# 定义专精技能的数据模型
class 训练任务(BaseModel):
    训练干员名称: str
    训练干员职业: str
    训练技能编号: str
    训练花费时间: int
    基于技能等级的效率编号: int


class 训练任务列表内容(BaseModel):
    训练位干员名称: str
    训练位干员职业: str
    训练位干员专精技能: str
    协助者进驻时间_艾丽妮: str
    协助者进驻时间: str
    协助者干员名: str
    元数据: str = "专精"


# 定义干员的数据模型


# 定义专精模型类
class 专精模型:
    def __init__(
        self,
        专精技能_json: List[Tuple[str, str, str, int, int]] = [
            ("第一个", "特种", "一技能", 0, 3),
            ("第二个", "近卫", "一技能", 0, 3),
            ("第二个", "近卫", "二技能", 0, 3),
        ],
        艾丽妮保底间隔: int = 3 * 60,
    ):
        self.专精列表: List[训练任务] = []

        self.艾丽妮 = 协助者(
            协助者干员名称="艾丽妮",
            协助职业=["近卫", "狙击"],
            协助效率=[30 + 5, 30 + 5, 30 + 5],
        )

        self.buff人 = 协助者(
            协助者干员名称="爱谁谁",
            协助职业=["近卫"],
            协助效率=[60 + 5, 60 + 5, 60 + 5],
        )
        self.专精基本时长 = (8 * 60 * 60, 16 * 60 * 60, 24 * 60 * 60)
        self.艾丽妮buff时间 = 5 * 60 * 60  # 5小时
        self.艾丽妮工作时间 = self.艾丽妮buff时间 + 艾丽妮保底间隔
        # 初始化专精列表
        for item in 专精技能_json:
            for i in range(item[3], item[4]):
                self.专精列表.append(
                    训练任务(
                        训练干员名称=item[0],
                        训练干员职业=item[1],
                        训练技能编号=item[2],
                        训练花费时间=self.专精基本时长[i],
                        基于技能等级的效率编号=i,
                    )
                )
        self.是否第一次专精 = True
        self.专精任务列表 = []

    def 计算艾丽妮buff时间(self, 职业: str) -> int:
        """
        根据职业计算艾丽妮的工作时间
        """
        if 职业 in self.艾丽妮.协助职业:
            return self.艾丽妮buff时间 * (1 + 0.3)
        return self.艾丽妮buff时间 * (1 + 0.05)

    def 计算工作时间(self, 花费时间: int, 是否第一次专精: bool) -> int:
        """
        根据是否第一次专精计算工作时间
        """
        if 是否第一次专精:
            return 花费时间
        return 花费时间 / 2

    def 计算其他干员工作时间(self, 剩余工作时间: int, item: 训练任务) -> int:
        """
        计算其他干员的工作时间
        """
        if 剩余工作时间 > 0:
            return (
                剩余工作时间
                * 100
                / (100 + self.buff人.协助效率[item.基于技能等级的效率编号])
            )
        return 0

    def 专精任务生成(self):
        """
        专精任务生成
        """
        current_干员名称 = self.专精列表[0].训练干员名称
        for item in self.专精列表:
            if current_干员名称 != item.训练干员名称:
                current_干员名称 = item.训练干员名称
                self.是否第一次专精 = True

            self.单次专精任务生成(item)

            self.是否第一次专精 = False
        print(self.专精任务列表)

    def 单次专精任务生成(self, item: 训练任务):
        总工作时间 = self.计算工作时间(item.训练花费时间, self.是否第一次专精)
        艾丽妮buff时间 = self.计算艾丽妮buff时间(item.训练干员职业)

        剩余工作时间 = 总工作时间 - 艾丽妮buff时间
        其他干员工作时间 = self.计算其他干员工作时间(剩余工作时间, item)

        专精技能列表内容实例 = 训练任务列表内容(
            训练位干员名称=item.训练干员名称,
            训练位干员职业=item.训练干员职业,
            训练位干员专精技能=item.训练技能编号,
            协助者进驻时间_艾丽妮=self.格式化时间(self.艾丽妮工作时间),
            协助者干员名=self.buff人.协助者干员名称,
            协助者进驻时间=self.格式化时间(其他干员工作时间),
        )
        self.专精任务列表.append(专精技能列表内容实例)

    def 格式化时间(self, 秒数: int) -> str:
        """
        使用 datetime 进行时间格式化
        """
        return (datetime.min + timedelta(seconds=秒数)).strftime("%H:%M:%S")
