"""
Copyright (c) 2024 Elaina <2901432375@qq.com>
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

This file incorporates work covered by the following copyright and
permission notice:

    Copyright (c) 2023 Funny_ppt <1763341376@qq.com>
    Copyright (c) 2023 Shawnsdaddy <wu2xx@dukes.jmu.edu>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""

import os
from pathlib import Path

appname = "mower"
appauthor = "ArkMower"
# global_space：多开时global_space为多开数据目录

global_space = None

# install_dir：源码目录

_install_dir = Path(os.getcwd()).resolve()


def get_path(path: str) -> Path:
    """
    使用 '@xxx/' 来表示一些特别的目录
    @app: 多开数据目录, 例如 get_path('@app/log/runtime.log')
    @install: 源码目录
    """
    global global_space
    path = path.replace("\\", "/")

    if isinstance(path, str) and path.startswith("@"):
        index = path.find("/")
        index = index if index != -1 else len(path)
        special_dir_name = path[1:index]
        relative_path = path[index:].strip("/")
        if special_dir_name == "app":
            return (
                Path(_install_dir) / relative_path
                if global_space is None
                else Path(global_space) / relative_path
            )
        elif special_dir_name == "install":
            return Path(_install_dir) / relative_path
        else:
            raise ValueError(
                "{}: {} 不是一个有效的特殊目录别名".format(path, special_dir_name)
            )
    else:
        return Path(path)
        # raise ValueError("{} 路径必须以 '@xxx' 开头".format(path))
