"""
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from mower import __system__, __version__
from mower.utils import typealias as tp
from mower.utils.log import logger

if __system__ == "windows" and __version__ == "7":
    from rapidocr_paddle import RapidOCR
else:
    from rapidocr_onnxruntime import RapidOCR

engine = None


def ocr_rec(img: tp.Image | tp.GrayImage) -> str | None:
    global engine
    if engine is None:
        engine = RapidOCR(text_score=0.3)

    result, elapse = engine(img, use_det=False, use_cls=False, use_rec=True)
    elapse = round(elapse[0] * 1000)
    try:
        text = result[0][0].strip()
    except Exception:
        text = None
    logger.debug(f"{elapse}ms {text}")
    return text
