"""
Copyright (c) 2024 Elaina <2901432375@qq.com>
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from mower.utils import typealias as tp


def va(a: tp.Coordinate, b: tp.Coordinate) -> tp.Coordinate:
    """向量加法，vector add"""
    return a[0] + b[0], a[1] + b[1]


def vs(a: tp.Coordinate, b: tp.Coordinate) -> tp.Coordinate:
    """向量减法，vector subtract"""
    return a[0] - b[0], a[1] - b[1]


def sa(scope: tp.Scope, vector: tp.Coordinate) -> tp.Scope:
    """区域偏移，scope add"""
    return va(scope[0], vector), va(scope[1], vector)


def sm(a: float, v: tp.Coordinate) -> tp.Coordinate:
    """数乘向量，scalar multiply"""
    return round(a * v[0]), round(a * v[1])


def ss(x_rate: float, y_rate: float, scope: tp.Scope) -> tp.Scope:
    """区域放缩，scope scaling"""
    return (
        int(scope[0][0] * x_rate),
        int(scope[0][1] * y_rate),
    ), (
        int(scope[1][0] * x_rate),
        int(scope[1][1] * y_rate),
    )


def in_scope(scope: tp.Scope, vector: tp.Coordinate) -> bool:
    """在区域内"""
    x, y = vector
    return scope[0][0] <= x <= scope[1][0] and scope[0][1] <= y <= scope[1][1]


def overlap(a: tp.Scope, b: tp.Scope) -> bool:
    """两个区域重叠"""
    (ax1, ay1), (ax2, ay2) = a
    (bx1, by1), (bx2, by2) = b
    return not (ax1 > bx2 or ax2 < bx1 or ay1 > by2 or ay2 < by1)


def clamp_vector(a: tp.Coordinate) -> tp.Coordinate:
    return max(0, min(a[0], 1920)), max(0, min(a[1], 1080))


def clamp_scope(a: tp.Scope) -> tp.Scope:
    return clamp_vector(a[0]), clamp_vector(a[1])
