import { IosArrowBack, IosArrowForward } from "./IosArrowForward.js";
import { Scrollbar } from "./Scrollbar.js";
import { derived, composite, c, cB, insideModal, insidePopover, cM, cE, defineComponent, h, useConfig, useRtl, useTheme, provide, toRef, computed, useThemeClass, createInjectionKey, inject, throwError, resolveSlot, ref, watchEffect, useMergedState, on, depx, off, call, onMounted, watch, createBlock, withCtx, openBlock, createVNode, createElementBlock, Fragment, renderList, unref, createTextVNode, toDisplayString, createBaseVNode, createCommentVNode, isRef, Button } from "./index.js";
import { __unplugin_components_2 } from "./Flex.js";
import { __unplugin_components_4, __unplugin_components_9 } from "./Image.js";
import { __unplugin_components_6 } from "./Slider.js";
import { NIcon } from "./Icon.js";
import "./index2.js";
import "./get-slot.js";
import "./utils.js";
import "./Tooltip.js";
import "./next-frame-once.js";
import "./use-compitable.js";
import "./use-locale.js";
import "./download.js";
function self$1(vars) {
  const {
    textColor2,
    cardColor,
    modalColor,
    popoverColor,
    dividerColor,
    borderRadius,
    fontSize,
    hoverColor
  } = vars;
  return {
    textColor: textColor2,
    color: cardColor,
    colorHover: hoverColor,
    colorModal: modalColor,
    colorHoverModal: composite(modalColor, hoverColor),
    colorPopover: popoverColor,
    colorHoverPopover: composite(popoverColor, hoverColor),
    borderColor: dividerColor,
    borderColorModal: composite(modalColor, dividerColor),
    borderColorPopover: composite(popoverColor, dividerColor),
    borderRadius,
    fontSize
  };
}
const listLight = {
  common: derived,
  self: self$1
};
const style$1 = c([cB("list", `
 --n-merged-border-color: var(--n-border-color);
 --n-merged-color: var(--n-color);
 --n-merged-color-hover: var(--n-color-hover);
 margin: 0;
 font-size: var(--n-font-size);
 transition:
 background-color .3s var(--n-bezier),
 color .3s var(--n-bezier),
 border-color .3s var(--n-bezier);
 padding: 0;
 list-style-type: none;
 color: var(--n-text-color);
 background-color: var(--n-merged-color);
 `, [cM("show-divider", [cB("list-item", [c("&:not(:last-child)", [cE("divider", `
 background-color: var(--n-merged-border-color);
 `)])])]), cM("clickable", [cB("list-item", `
 cursor: pointer;
 `)]), cM("bordered", `
 border: 1px solid var(--n-merged-border-color);
 border-radius: var(--n-border-radius);
 `), cM("hoverable", [cB("list-item", `
 border-radius: var(--n-border-radius);
 `, [c("&:hover", `
 background-color: var(--n-merged-color-hover);
 `, [cE("divider", `
 background-color: transparent;
 `)])])]), cM("bordered, hoverable", [cB("list-item", `
 padding: 12px 20px;
 `), cE("header, footer", `
 padding: 12px 20px;
 `)]), cE("header, footer", `
 padding: 12px 0;
 box-sizing: border-box;
 transition: border-color .3s var(--n-bezier);
 `, [c("&:not(:last-child)", `
 border-bottom: 1px solid var(--n-merged-border-color);
 `)]), cB("list-item", `
 position: relative;
 padding: 12px 0; 
 box-sizing: border-box;
 display: flex;
 flex-wrap: nowrap;
 align-items: center;
 transition:
 background-color .3s var(--n-bezier),
 border-color .3s var(--n-bezier);
 `, [cE("prefix", `
 margin-right: 20px;
 flex: 0;
 `), cE("suffix", `
 margin-left: 20px;
 flex: 0;
 `), cE("main", `
 flex: 1;
 `), cE("divider", `
 height: 1px;
 position: absolute;
 bottom: 0;
 left: 0;
 right: 0;
 background-color: transparent;
 transition: background-color .3s var(--n-bezier);
 pointer-events: none;
 `)])]), insideModal(cB("list", `
 --n-merged-color-hover: var(--n-color-hover-modal);
 --n-merged-color: var(--n-color-modal);
 --n-merged-border-color: var(--n-border-color-modal);
 `)), insidePopover(cB("list", `
 --n-merged-color-hover: var(--n-color-hover-popover);
 --n-merged-color: var(--n-color-popover);
 --n-merged-border-color: var(--n-border-color-popover);
 `))]);
const listProps = Object.assign(Object.assign({}, useTheme.props), {
  size: {
    type: String,
    default: "medium"
  },
  bordered: Boolean,
  clickable: Boolean,
  hoverable: Boolean,
  showDivider: {
    type: Boolean,
    default: true
  }
});
const listInjectionKey = createInjectionKey("n-list");
const __unplugin_components_1 = defineComponent({
  name: "List",
  props: listProps,
  slots: Object,
  setup(props) {
    const {
      mergedClsPrefixRef,
      inlineThemeDisabled,
      mergedRtlRef
    } = useConfig(props);
    const rtlEnabledRef = useRtl("List", mergedRtlRef, mergedClsPrefixRef);
    const themeRef = useTheme("List", "-list", style$1, listLight, props, mergedClsPrefixRef);
    provide(listInjectionKey, {
      showDividerRef: toRef(props, "showDivider"),
      mergedClsPrefixRef
    });
    const cssVarsRef = computed(() => {
      const {
        common: {
          cubicBezierEaseInOut
        },
        self: {
          fontSize,
          textColor,
          color,
          colorModal,
          colorPopover,
          borderColor,
          borderColorModal,
          borderColorPopover,
          borderRadius,
          colorHover,
          colorHoverModal,
          colorHoverPopover
        }
      } = themeRef.value;
      return {
        "--n-font-size": fontSize,
        "--n-bezier": cubicBezierEaseInOut,
        "--n-text-color": textColor,
        "--n-color": color,
        "--n-border-radius": borderRadius,
        "--n-border-color": borderColor,
        "--n-border-color-modal": borderColorModal,
        "--n-border-color-popover": borderColorPopover,
        "--n-color-modal": colorModal,
        "--n-color-popover": colorPopover,
        "--n-color-hover": colorHover,
        "--n-color-hover-modal": colorHoverModal,
        "--n-color-hover-popover": colorHoverPopover
      };
    });
    const themeClassHandle = inlineThemeDisabled ? useThemeClass("list", void 0, cssVarsRef, props) : void 0;
    return {
      mergedClsPrefix: mergedClsPrefixRef,
      rtlEnabled: rtlEnabledRef,
      cssVars: inlineThemeDisabled ? void 0 : cssVarsRef,
      themeClass: themeClassHandle === null || themeClassHandle === void 0 ? void 0 : themeClassHandle.themeClass,
      onRender: themeClassHandle === null || themeClassHandle === void 0 ? void 0 : themeClassHandle.onRender
    };
  },
  render() {
    var _a;
    const {
      $slots,
      mergedClsPrefix,
      onRender
    } = this;
    onRender === null || onRender === void 0 ? void 0 : onRender();
    return h("ul", {
      class: [`${mergedClsPrefix}-list`, this.rtlEnabled && `${mergedClsPrefix}-list--rtl`, this.bordered && `${mergedClsPrefix}-list--bordered`, this.showDivider && `${mergedClsPrefix}-list--show-divider`, this.hoverable && `${mergedClsPrefix}-list--hoverable`, this.clickable && `${mergedClsPrefix}-list--clickable`, this.themeClass],
      style: this.cssVars
    }, $slots.header ? h("div", {
      class: `${mergedClsPrefix}-list__header`
    }, $slots.header()) : null, (_a = $slots.default) === null || _a === void 0 ? void 0 : _a.call($slots), $slots.footer ? h("div", {
      class: `${mergedClsPrefix}-list__footer`
    }, $slots.footer()) : null);
  }
});
const __unplugin_components_0 = defineComponent({
  name: "ListItem",
  slots: Object,
  setup() {
    const listInjection = inject(listInjectionKey, null);
    if (!listInjection) {
      throwError("list-item", "`n-list-item` must be placed in `n-list`.");
    }
    return {
      showDivider: listInjection.showDividerRef,
      mergedClsPrefix: listInjection.mergedClsPrefixRef
    };
  },
  render() {
    const {
      $slots,
      mergedClsPrefix
    } = this;
    return h("li", {
      class: `${mergedClsPrefix}-list-item`
    }, $slots.prefix ? h("div", {
      class: `${mergedClsPrefix}-list-item__prefix`
    }, $slots.prefix()) : null, $slots.default ? h("div", {
      class: `${mergedClsPrefix}-list-item__main`
    }, $slots) : null, $slots.suffix ? h("div", {
      class: `${mergedClsPrefix}-list-item__suffix`
    }, $slots.suffix()) : null, this.showDivider && h("div", {
      class: `${mergedClsPrefix}-list-item__divider`
    }));
  }
});
function self(vars) {
  const {
    primaryColorHover,
    borderColor
  } = vars;
  return {
    resizableTriggerColorHover: primaryColorHover,
    resizableTriggerColor: borderColor
  };
}
const themeLight = {
  common: derived,
  self
};
const style = cB("split", `
 display: flex;
 width: 100%;
 height: 100%;
`, [cM("horizontal", `
 flex-direction: row;
 `), cM("vertical", `
 flex-direction: column;
 `), cB("split-pane-1", `
 overflow: hidden;
 `), cB("split-pane-2", `
 overflow: hidden;
 flex: 1;
 `), cE("resize-trigger", `
 background-color: var(--n-resize-trigger-color);
 transition: background-color .3s var(--n-bezier);
 `, [cM("hover", `
 background-color: var(--n-resize-trigger-color-hover);
 `), c("&:hover", `
 background-color: var(--n-resize-trigger-color-hover);
 `)])]);
const splitProps = Object.assign(Object.assign({}, useTheme.props), {
  direction: {
    type: String,
    default: "horizontal"
  },
  resizeTriggerSize: {
    type: Number,
    default: 3
  },
  disabled: Boolean,
  defaultSize: {
    type: [String, Number],
    default: 0.5
  },
  "onUpdate:size": [Function, Array],
  onUpdateSize: [Function, Array],
  size: [String, Number],
  min: {
    type: [String, Number],
    default: 0
  },
  max: {
    type: [String, Number],
    default: 1
  },
  pane1Class: String,
  pane1Style: [Object, String],
  pane2Class: String,
  pane2Style: [Object, String],
  onDragStart: Function,
  onDragMove: Function,
  onDragEnd: Function,
  watchProps: Array
});
const __unplugin_components_5 = defineComponent({
  name: "Split",
  props: splitProps,
  slots: Object,
  setup(props) {
    var _a;
    const {
      mergedClsPrefixRef,
      inlineThemeDisabled
    } = useConfig(props);
    const themeRef = useTheme("Split", "-split", style, themeLight, props, mergedClsPrefixRef);
    const cssVarsRef = computed(() => {
      const {
        common: {
          cubicBezierEaseInOut
        },
        self: {
          resizableTriggerColor,
          resizableTriggerColorHover
        }
      } = themeRef.value;
      return {
        "--n-bezier": cubicBezierEaseInOut,
        "--n-resize-trigger-color": resizableTriggerColor,
        "--n-resize-trigger-color-hover": resizableTriggerColorHover
      };
    });
    const resizeTriggerElRef = ref(null);
    const isDraggingRef = ref(false);
    const controlledSizeRef = toRef(props, "size");
    const uncontrolledSizeRef = ref(props.defaultSize);
    if ((_a = props.watchProps) === null || _a === void 0 ? void 0 : _a.includes("defaultSize")) {
      watchEffect(() => uncontrolledSizeRef.value = props.defaultSize);
    }
    const doUpdateSize = (size) => {
      const _onUpdateSize = props["onUpdate:size"];
      if (props.onUpdateSize) call(props.onUpdateSize, size);
      if (_onUpdateSize) call(_onUpdateSize, size);
      uncontrolledSizeRef.value = size;
    };
    const mergedSizeRef = useMergedState(controlledSizeRef, uncontrolledSizeRef);
    const firstPaneStyle = computed(() => {
      const sizeValue = mergedSizeRef.value;
      if (typeof sizeValue === "string") {
        return {
          flex: `0 0 ${sizeValue}`
        };
      } else if (typeof sizeValue === "number") {
        const size = sizeValue * 100;
        return {
          flex: `0 0 calc(${size}% - ${props.resizeTriggerSize * size / 100}px)`
        };
      }
    });
    const resizeTriggerStyle = computed(() => {
      return props.direction === "horizontal" ? {
        width: `${props.resizeTriggerSize}px`,
        height: "100%"
      } : {
        width: "100%",
        height: `${props.resizeTriggerSize}px`
      };
    });
    const resizeTriggerWrapperStyle = computed(() => {
      const horizontal = props.direction === "horizontal";
      return {
        width: horizontal ? `${props.resizeTriggerSize}px` : "",
        height: horizontal ? "" : `${props.resizeTriggerSize}px`,
        cursor: props.direction === "horizontal" ? "col-resize" : "row-resize"
      };
    });
    let offset = 0;
    const handleMouseDown = (e) => {
      e.preventDefault();
      isDraggingRef.value = true;
      if (props.onDragStart) props.onDragStart(e);
      const mouseMoveEvent = "mousemove";
      const mouseUpEvent = "mouseup";
      const onMouseMove = (e2) => {
        updateSize(e2);
        if (props.onDragMove) props.onDragMove(e2);
      };
      const onMouseUp = () => {
        off(mouseMoveEvent, document, onMouseMove);
        off(mouseUpEvent, document, onMouseUp);
        isDraggingRef.value = false;
        if (props.onDragEnd) props.onDragEnd(e);
        document.body.style.cursor = "";
      };
      document.body.style.cursor = resizeTriggerWrapperStyle.value.cursor;
      on(mouseMoveEvent, document, onMouseMove);
      on(mouseUpEvent, document, onMouseUp);
      const resizeTriggerEl = resizeTriggerElRef.value;
      if (resizeTriggerEl) {
        const elRect = resizeTriggerEl.getBoundingClientRect();
        if (props.direction === "horizontal") {
          offset = e.clientX - elRect.left;
        } else {
          offset = elRect.top - e.clientY;
        }
      }
      updateSize(e);
    };
    function updateSize(event) {
      var _a2, _b;
      const containerRect = (_b = (_a2 = resizeTriggerElRef.value) === null || _a2 === void 0 ? void 0 : _a2.parentElement) === null || _b === void 0 ? void 0 : _b.getBoundingClientRect();
      if (!containerRect) return;
      const {
        direction
      } = props;
      const containerUsableWidth = containerRect.width - props.resizeTriggerSize;
      const containerUsableHeight = containerRect.height - props.resizeTriggerSize;
      const containerUsableSize = direction === "horizontal" ? containerUsableWidth : containerUsableHeight;
      const newPxSize = direction === "horizontal" ? event.clientX - containerRect.left - offset : event.clientY - containerRect.top + offset;
      const {
        min,
        max
      } = props;
      const pxMin = typeof min === "string" ? depx(min) : min * containerUsableSize;
      const pxMax = typeof max === "string" ? depx(max) : max * containerUsableSize;
      let nextPxSize = newPxSize;
      nextPxSize = Math.max(nextPxSize, pxMin);
      nextPxSize = Math.min(nextPxSize, pxMax, containerUsableSize);
      if (typeof mergedSizeRef.value === "string") {
        doUpdateSize(`${nextPxSize}px`);
      } else {
        doUpdateSize(nextPxSize / containerUsableSize);
      }
    }
    const themeClassHandle = inlineThemeDisabled ? useThemeClass("split", void 0, cssVarsRef, props) : void 0;
    return {
      themeClass: themeClassHandle === null || themeClassHandle === void 0 ? void 0 : themeClassHandle.themeClass,
      onRender: themeClassHandle === null || themeClassHandle === void 0 ? void 0 : themeClassHandle.onRender,
      cssVars: inlineThemeDisabled ? void 0 : cssVarsRef,
      resizeTriggerElRef,
      isDragging: isDraggingRef,
      mergedClsPrefix: mergedClsPrefixRef,
      resizeTriggerWrapperStyle,
      resizeTriggerStyle,
      handleMouseDown,
      firstPaneStyle
    };
  },
  render() {
    var _a, _b, _c, _d, _e;
    (_a = this.onRender) === null || _a === void 0 ? void 0 : _a.call(this);
    return h("div", {
      class: [`${this.mergedClsPrefix}-split`, `${this.mergedClsPrefix}-split--${this.direction}`, this.themeClass],
      style: this.cssVars
    }, h("div", {
      class: [`${this.mergedClsPrefix}-split-pane-1`, this.pane1Class],
      style: [this.firstPaneStyle, this.pane1Style]
    }, (_c = (_b = this.$slots)[1]) === null || _c === void 0 ? void 0 : _c.call(_b)), !this.disabled && h("div", {
      ref: "resizeTriggerElRef",
      class: `${this.mergedClsPrefix}-split__resize-trigger-wrapper`,
      style: this.resizeTriggerWrapperStyle,
      onMousedown: this.handleMouseDown
    }, resolveSlot(this.$slots["resize-trigger"], () => [h("div", {
      style: this.resizeTriggerStyle,
      class: [`${this.mergedClsPrefix}-split__resize-trigger`, this.isDragging && `${this.mergedClsPrefix}-split__resize-trigger--hover`]
    })])), h("div", {
      class: [`${this.mergedClsPrefix}-split-pane-2`, this.pane2Class],
      style: this.pane2Style
    }, (_e = (_d = this.$slots)[2]) === null || _e === void 0 ? void 0 : _e.call(_d)));
  }
});
const _hoisted_1 = { style: { "text-align": "center", "margin": "0" } };
const _hoisted_2 = { style: { "margin": "0", "white-space": "pre-wrap" } };
const _sfc_main = {
  __name: "DebugConsole",
  setup(__props) {
    const axios = inject("axios");
    const log_list = ref([]);
    const current_log = ref("");
    const log_content = ref([]);
    const log_frame = ref(0);
    const log_scroll = ref(null);
    onMounted(() => {
      axios.get(`${""}/debug/log`).then(({ data }) => {
        log_list.value = data;
      });
      get_log("runtime.log");
    });
    function get_log(log) {
      current_log.value = log;
      axios.get(`${""}/debug/log/${log}`).then(({ data }) => {
        log_frame.value = 0;
        log_content.value = data;
        log_frame.value = log_content.value.length - 1;
      });
    }
    const screenshot = computed(() => {
      var _a;
      return (_a = log_content.value[log_frame.value]) == null ? void 0 : _a.screenshot;
    });
    const image_url = computed(() => {
      return `${""}/debug/screenshot/${screenshot.value}`;
    });
    watch(log_frame, () => {
      var _a;
      (_a = log_scroll.value) == null ? void 0 : _a.scrollTo({ top: 0 });
    });
    return (_ctx, _cache) => {
      const _component_n_list_item = __unplugin_components_0;
      const _component_n_list = __unplugin_components_1;
      const _component_n_scrollbar = Scrollbar;
      const _component_n_flex = __unplugin_components_2;
      const _component_n_image = __unplugin_components_4;
      const _component_n_split = __unplugin_components_5;
      const _component_n_slider = __unplugin_components_6;
      const _component_n_icon = NIcon;
      const _component_n_button = Button;
      const _component_n_button_group = __unplugin_components_9;
      return openBlock(), createBlock(_component_n_flex, { style: { "width": "100%", "height": "100%", "gap": "0", "overflow-y": "hidden" } }, {
        default: withCtx(() => [
          createVNode(_component_n_scrollbar, { style: { "width": "220px" } }, {
            default: withCtx(() => [
              createVNode(_component_n_list, {
                hoverable: "",
                clickable: "",
                bordered: ""
              }, {
                default: withCtx(() => [
                  (openBlock(true), createElementBlock(Fragment, null, renderList(unref(log_list), (log) => {
                    return openBlock(), createBlock(_component_n_list_item, {
                      onClick: ($event) => get_log(log)
                    }, {
                      default: withCtx(() => [
                        createTextVNode(toDisplayString(log), 1)
                      ]),
                      _: 2
                    }, 1032, ["onClick"]);
                  }), 256))
                ]),
                _: 1
              })
            ]),
            _: 1
          }),
          createVNode(_component_n_flex, {
            vertical: "",
            style: { "flex": "1", "height": "100%", "overflow": "hidden" }
          }, {
            default: withCtx(() => [
              createVNode(_component_n_flex, {
                justify: "center",
                align: "center",
                style: { "margin-top": "8px" }
              }, {
                default: withCtx(() => {
                  var _a;
                  return [
                    createBaseVNode("h2", _hoisted_1, toDisplayString(unref(current_log)) + " / " + toDisplayString((_a = unref(log_content)[unref(log_frame)]) == null ? void 0 : _a.time), 1)
                  ];
                }),
                _: 1
              }),
              createVNode(_component_n_split, {
                direction: "vertical",
                style: { "overflow": "hidden", "gap": "4px" }
              }, {
                "1": withCtx(() => [
                  unref(screenshot) ? (openBlock(), createBlock(_component_n_image, {
                    key: 0,
                    src: unref(image_url),
                    alt: unref(screenshot),
                    width: "100%",
                    "object-fit": "scale-down"
                  }, null, 8, ["src", "alt"])) : createCommentVNode("", true)
                ]),
                "2": withCtx(() => [
                  createVNode(_component_n_scrollbar, {
                    ref_key: "log_scroll",
                    ref: log_scroll,
                    style: { "width": "100%" },
                    trigger: "none",
                    "content-style": "font-size: 16px; user-select: text; padding: 0 12px"
                  }, {
                    default: withCtx(() => {
                      var _a;
                      return [
                        (openBlock(true), createElementBlock(Fragment, null, renderList((_a = unref(log_content)[unref(log_frame)]) == null ? void 0 : _a.log, (line) => {
                          return openBlock(), createElementBlock("div", null, [
                            createBaseVNode("pre", _hoisted_2, [
                              createBaseVNode("code", null, toDisplayString(line), 1)
                            ])
                          ]);
                        }), 256))
                      ];
                    }),
                    _: 1
                  }, 512)
                ]),
                _: 1
              }),
              createVNode(_component_n_flex, {
                align: "center",
                style: { "padding": "0 64px 6px 12px" }
              }, {
                default: withCtx(() => [
                  createVNode(_component_n_slider, {
                    style: { "flex": "1" },
                    value: unref(log_frame),
                    "onUpdate:value": _cache[0] || (_cache[0] = ($event) => isRef(log_frame) ? log_frame.value = $event : null),
                    min: 0,
                    max: unref(log_content).length - 1,
                    "format-tooltip": (idx) => {
                      var _a;
                      return (_a = unref(log_content)[idx]) == null ? void 0 : _a.time;
                    }
                  }, null, 8, ["value", "max", "format-tooltip"]),
                  createVNode(_component_n_button_group, null, {
                    default: withCtx(() => [
                      createVNode(_component_n_button, {
                        disabled: unref(log_frame) == 0,
                        onClick: _cache[1] || (_cache[1] = ($event) => log_frame.value--)
                      }, {
                        icon: withCtx(() => [
                          createVNode(_component_n_icon, null, {
                            default: withCtx(() => [
                              createVNode(unref(IosArrowBack))
                            ]),
                            _: 1
                          })
                        ]),
                        _: 1
                      }, 8, ["disabled"]),
                      createVNode(_component_n_button, {
                        disabled: unref(log_frame) == unref(log_content).length - 1,
                        onClick: _cache[2] || (_cache[2] = ($event) => log_frame.value++)
                      }, {
                        icon: withCtx(() => [
                          createVNode(_component_n_icon, null, {
                            default: withCtx(() => [
                              createVNode(unref(IosArrowForward))
                            ]),
                            _: 1
                          })
                        ]),
                        _: 1
                      }, 8, ["disabled"])
                    ]),
                    _: 1
                  })
                ]),
                _: 1
              })
            ]),
            _: 1
          })
        ]),
        _: 1
      });
    };
  }
};
export {
  _sfc_main as default
};
