import { useRecordStore } from "./record.js";
import { ECharts } from "./index3.js";
import { _export_sfc, ref, shallowRef, onMounted, onBeforeUnmount, createElementBlock, createBaseVNode, createVNode, withCtx, isRef, unref, __unplugin_components_4, openBlock, Fragment, renderList, createBlock, __unplugin_components_0, createCommentVNode, normalizeStyle, Button, createTextVNode } from "./index.js";
import { use, install, install$1, install$2, install$3, install$4, install$5, install$6 } from "./install.js";
import { __unplugin_components_3, __unplugin_components_2 } from "./Grid.js";
import "./get-slot.js";
import "./next-frame-once.js";
const _sfc_main = {
  __name: "RecordLine",
  setup(__props) {
    const chartHeight = ref(300);
    const modalVisible = ref(false);
    const selectedIndex = ref(null);
    use([
      install,
      install$1,
      install$2,
      install$3,
      install$4,
      install$5,
      install$6
    ]);
    const recordStore = useRecordStore();
    const { getMoodRatios } = recordStore;
    const reportData = shallowRef([]);
    const chartOptions = shallowRef([]);
    const mounted = ref(false);
    const expand_chart = ref([]);
    const openModal = (index) => {
      selectedIndex.value = index;
      modalVisible.value = true;
    };
    const baseOptions = {
      tooltip: {
        trigger: "axis",
        confine: true
      },
      grid: {
        top: 30,
        bottom: 30,
        left: 40,
        right: 20
      },
      xAxis: {
        type: "time",
        axisLabel: {
          formatter: "{M}/{d}"
        }
      },
      yAxis: {
        type: "value",
        min: 0,
        max: 24,
        interval: 4
      },
      dataZoom: [
        {
          type: "inside",
          start: 0,
          end: 100
        }
      ]
    };
    onMounted(async () => {
      const data = await getMoodRatios();
      reportData.value = data;
      expand_chart.value = new Array(data.length).fill(100);
      chartOptions.value = data.map((group) => ({
        ...baseOptions,
        legend: {
          show: true,
          // 显示图例
          data: group.moodData.datasets.map((dataset) => dataset.label)
          // 图例数据
        },
        series: group.moodData.datasets.map((dataset) => ({
          name: dataset.label,
          type: "line",
          showSymbol: false,
          smooth: true,
          data: dataset.data.map((item) => [
            typeof item.x === "string" ? +new Date(item.x) : item.x,
            item.y
          ]),
          lineStyle: { width: 1.5 }
        }))
      }));
      mounted.value = true;
    });
    onBeforeUnmount(() => {
      mounted.value = false;
    });
    return (_ctx, _cache) => {
      const _component_n_button = Button;
      const _component_n_card = __unplugin_components_0;
      const _component_n_gi = __unplugin_components_2;
      const _component_n_grid = __unplugin_components_3;
      const _component_n_modal = __unplugin_components_4;
      return openBlock(), createElementBlock("div", null, [
        _cache[2] || (_cache[2] = createBaseVNode("h1", null, "干员心情折线表", -1)),
        createVNode(_component_n_grid, {
          "x-gap": 12,
          "y-gap": 8,
          collapsed: false,
          cols: "1 s:1 m:2 l:3 xl:4 2xl:5",
          responsive: "screen"
        }, {
          default: withCtx(() => [
            (openBlock(true), createElementBlock(Fragment, null, renderList(reportData.value, (groupData, index) => {
              return openBlock(), createBlock(_component_n_gi, {
                key: groupData.groupName
              }, {
                default: withCtx(() => [
                  createVNode(_component_n_card, {
                    title: groupData.groupName
                  }, {
                    "header-extra": withCtx(() => [
                      createVNode(_component_n_button, {
                        onClick: ($event) => openModal(index)
                      }, {
                        default: withCtx(() => _cache[1] || (_cache[1] = [
                          createTextVNode("全屏查看")
                        ])),
                        _: 2
                      }, 1032, ["onClick"])
                    ]),
                    default: withCtx(() => [
                      unref(mounted) ? (openBlock(), createBlock(unref(ECharts), {
                        key: 0,
                        option: chartOptions.value[index],
                        style: normalizeStyle({ height: unref(chartHeight) + "px", width: unref(chartHeight) + "px" }),
                        autoresize: ""
                      }, null, 8, ["option", "style"])) : createCommentVNode("", true)
                    ]),
                    _: 2
                  }, 1032, ["title"])
                ]),
                _: 2
              }, 1024);
            }), 128))
          ]),
          _: 1
        }),
        createVNode(_component_n_modal, {
          show: unref(modalVisible),
          "onUpdate:show": _cache[0] || (_cache[0] = ($event) => isRef(modalVisible) ? modalVisible.value = $event : null),
          style: { "max-width": "80vw", "height": "80vh" }
        }, {
          default: withCtx(() => {
            var _a;
            return [
              unref(selectedIndex) !== null ? (openBlock(), createBlock(_component_n_card, {
                key: 0,
                title: (_a = reportData.value[unref(selectedIndex)]) == null ? void 0 : _a.groupName
              }, {
                default: withCtx(() => [
                  createVNode(unref(ECharts), {
                    option: chartOptions.value[unref(selectedIndex)],
                    style: { "max-width": "80vw", "height": "80vh" }
                  }, null, 8, ["option"])
                ]),
                _: 1
              }, 8, ["title"])) : createCommentVNode("", true)
            ];
          }),
          _: 1
        }, 8, ["show"])
      ]);
    };
  }
};
const RecordLine = /* @__PURE__ */ _export_sfc(_sfc_main, [["__scopeId", "data-v-b9b10beb"]]);
export {
  RecordLine as default
};
