// ui/src/utils/hljs.js
import hljs from 'highlight.js/lib/core'
import json from 'highlight.js/lib/languages/json'

// 注册 JSON 语言
hljs.registerLanguage('json', json)

/**
 * 自定义 'mower' 语言
 * @param {Array} operators -
 * @returns {void}
 */
export function registerMowerLanguage(operators) {
  // 构建正则表达式，匹配操作员名称
  const r = RegExp(operators.map((x) => "'" + x.value).join('|'))

  // 注册自定义的 'mower' 语言
  hljs.registerLanguage('mower', () => ({
    contains: [
      {
        begin: r,
        end: /'/,
        className: 'operator',
        relevance: 0
      },
      {
        begin: /宿舍黑名单|重设上次房间为空/,
        relevance: 10
      },
      {
        begin: /[0-9]+(-[0-9]+)+/,
        className: 'date'
      },
      {
        begin: /[0-9]+:[0-9]+:[0-9]+((\.|,)[0-9]+)?/,
        className: 'time'
      },
      {
        begin: /room_[0-9]_[0-9]|dormitory_[0-9]|central|contact|factory|meeting/,
        className: 'room'
      },
      {
        begin: /INFO/,
        className: 'info'
      },
      {
        begin: /WARNING/,
        className: 'warning'
      },
      {
        begin: /ERROR/,
        className: 'error'
      },
      {
        begin: /Scene [0-9]+:.*/,
        className: 'scene'
      }
    ]
  }))
}

// 导出 hljs 实例
export default hljs
